// ======================================================================
//
// MenuHover.cpp
// hugh david & ~Chuck Karpiak
//
// copyright 1997 origin systems inc
//
// ======================================================================

#include "First.h"
#include "MenuHover.h"

#include "Iff.h"
#include "Mouse.h"

// ======================================================================

const Tag TAG_HVEV = TAG(H,V,E,V);

// ======================================================================

MenuHover::MenuHover(MenuElement *newParent, int maxWidth, int maxHeight, Iff *iff)
: MenuElement(newParent, maxWidth, maxHeight, NULL)
{
	init();
	load(iff);
}

// ----------------------------------------------------------------------

MenuHover::~MenuHover(void)
{
	delete []events;
	init();
}

// ----------------------------------------------------------------------

void MenuHover::init(void)
{
	events        = NULL;
	numEvents     = 0;
	mouseIsOverMe = false;
}

// ----------------------------------------------------------------------

void MenuHover::load(Iff *iff)
{
	int i;
	if (iff)
	{
		MenuElement::load(iff);

		// we may want multiple events to occur when hover'd over
		numEvents = iff->blockCount(Iff::FlagChunk, TAG_HVEV);
		if (numEvents > 0)
		{
			events = NEW Event[numEvents];
			memset(events, '\0', sizeof(*events) * numEvents);
			for (i = 0; iff->seekChunk(TAG_HVEV, Iff::FlagOptional); i++)
			{
				events[i].start = iff->read_int32();
				events[i].stop  = iff->read_int32();
				iff->exitChunk();
			}
		}
	}
}

// ----------------------------------------------------------------------
// hover event messages occur only once for each 'start' and 'stop'

IoResult MenuHover::event(IoEventStruct *event)
{
	int i;

	if (menuRect.isInInclusive(event->mouseX, event->mouseY) && event->type == IoEventMouseSetCursor)
	{
		for (i = 0; !mouseIsOverMe && i < numEvents; i++)
			MenuElement::menuEvent(events[i].start, this);
		mouseIsOverMe = true;
	}

	return MenuElement::event(event);
}

// ----------------------------------------------------------------------
// this used to be in the event, but there was a problem with the user being
//  able to get out of the hover region without the hover region getting the
//  "just went outside of rect" event.

IoResult MenuHover::update(void)
{
	Rect r;
	int i;

//	if (mouseIsOverMe)
	{
		getAbsoluteRect(&r);
		if (!r.isInInclusive(Mouse::getX(), Mouse::getY()))
		{
			for (i = 0; i < numEvents; i++)
				MenuElement::menuEvent(events[i].stop, this);
			mouseIsOverMe = false;
		}
	}

	return MenuElement::update();
}

// ----------------------------------------------------------------------

void MenuHover::menuDraw(int x, int y) const
{
	MenuElement::menuDraw(x, y);
}

// ======================================================================
